@@ -15,9 +15,13 @@
 #
 # You should have received a copy of the GNU Lesser General Public License
 # along with Lightbulb. If not, see <https://www.gnu.org/licenses/>.
-from lightbulb.command_handler import BotWithHandler as Bot
+from lightbulb.command_handler import Bot
 from lightbulb import errors
 from lightbulb import plugins
+from lightbulb import commands
+from lightbulb import converters
+from lightbulb import cooldowns
 from lightbulb.checks import guild_only, dm_only, owner_only
+from lightbulb import utils
 
-__version__ = "0.0.15"
+__version__ = "0.0.26"
