@@ -23,6 +23,7 @@ import collections
 import inspect
 import sys
 import importlib
+import logging
 
 import hikari
 from hikari.events import message
@@ -33,11 +34,15 @@ from lightbulb import context
 from lightbulb import errors
 from lightbulb import stringview
 from lightbulb import plugins
-from lightbulb import help
+from lightbulb import help as help_command
 
 if typing.TYPE_CHECKING:
     from hikari.models import messages
 
+__all__: typing.Final[typing.Tuple[str]] = ("Bot",)
+
+_LOGGER = logging.getLogger("lightbulb")
+
 
 async def _return_prefix(
     _, __, *, prefixes: typing.Union[str, typing.List[str], typing.Tuple[str]]
@@ -45,7 +50,7 @@ async def _return_prefix(
     return prefixes
 
 
-class BotWithHandler(hikari.Bot):
+class Bot(hikari.Bot):
     """
     A subclassed implementation of :class:`hikari.impl.bot.BotAppImpl` which contains a command handler.
     This should be instantiated instead of the superclass if you want to be able to use
@@ -60,34 +65,27 @@ class BotWithHandler(hikari.Bot):
     - A function or coroutine that takes **only** two arguments, ``bot`` and ``message``, and that returns a single string or iterable of strings.
 
     Args:
-        prefix: The bot's command prefix, iterable of prefixes, or callable that returns a prefix or iterable of prefixes.
-        insensitive_commands (:obj:`bool`): Whether or not commands should be case insensitive. Defaults to ``False``.
-        ignore_bots (:obj:`bool`): Whether or not the bot should ignore its commands when invoked by other bots. Defaults to ``True``.
+        prefix: The bot's command prefix, iterable of prefixes, or callable that returns
+            a prefix or iterable of prefixes.
+        insensitive_commands (:obj:`bool`): Whether or not commands should be case-insensitive or not.
+            Defaults to False (commands are case-sensitive).
+        ignore_bots (:obj:`bool`): Ignore other bot's messages invoking your bot's commands if True (default), else not.
+            invoked by other bots. Defaults to ``True``.
         owner_ids (List[ :obj:`int` ]): IDs that the bot should treat as owning the bot.
-        help_class (:obj:`~.help.HelpCommand`): The **uninstantiated** class the bot should use for it's help command. Defaults
-            to :obj:`~.help.HelpCommand`. Any class passed should always be this class or subclass of this class.
+        help_class (:obj:`~.help.HelpCommand`): The **uninstantiated** class the bot should use for it's help command.
+            Defaults to :obj:`~.help.HelpCommand`. Any class passed should always be this class or subclass of
+            this class.
         **kwargs: Other parameters passed to the :class:`hikari.impl.bot.BotAppImpl` constructor.
     """
 
     def __init__(
         self,
         *,
-        prefix: typing.Union[
-            typing.Iterable[str],
-            typing.Callable[
-                [BotWithHandler, messages.Message],
-                typing.Union[
-                    typing.Callable[
-                        [BotWithHandler, messages.Message], typing.Iterable[str],
-                    ],
-                    typing.Coroutine[None, typing.Any, typing.Iterable[str]],
-                ],
-            ],
-        ],
+        prefix,
         insensitive_commands: bool = False,
         ignore_bots: bool = True,
         owner_ids: typing.Iterable[int] = (),
-        help_class=help.HelpCommand,
+        help_class: typing.Type[help_command.HelpCommand] = help_command.HelpCommand,
         **kwargs,
     ) -> None:
         super().__init__(**kwargs)
@@ -114,7 +112,7 @@ class BotWithHandler(hikari.Bot):
     async def fetch_owner_ids(self) -> None:
         """
         Fetches the IDs of the bot's owner(s) from the API and stores them in
-        :attr:`~.command_handler.BotWithHandler.owner_ids`
+        :attr:`~.command_handler.Bot.owner_ids`
 
         Returns:
             ``None``
@@ -145,22 +143,11 @@ class BotWithHandler(hikari.Bot):
                     await ctx.reply("Pong!")
 
         See Also:
-            :meth:`~.command_handler.BotWithHandler.add_command`
+            :meth:`~.command_handler.Bot.add_command`
         """
-        registered_commands = self.commands
 
         def decorate(func: typing.Callable):
-            nonlocal registered_commands
-            name = kwargs.get("name", func.__name__)
-            registered_commands[name] = commands.Command(
-                func,
-                name,
-                kwargs.get("allow_extra_arguments", True),
-                kwargs.get("aliases", []),
-            )
-            for alias in kwargs.get("aliases", []):
-                registered_commands[alias] = registered_commands[name]
-            return registered_commands[name]
+            return self.add_command(func, **kwargs)
 
         return decorate
 
@@ -184,30 +171,18 @@ class BotWithHandler(hikari.Bot):
         See Also:
             :meth:`~.commands.Group.command` for how to add subcommands to a group.
         """
-        registered_commands = self.commands
 
         def decorate(func: typing.Callable):
-            nonlocal registered_commands
-            name = kwargs.get("name", func.__name__)
-            registered_commands[name] = commands.Group(
-                func,
-                name,
-                kwargs.get("allow_extra_arguments", True),
-                kwargs.get("aliases", []),
-                insensitive_commands=kwargs.get("insensitive_commands", False),
-            )
-            for alias in kwargs.get("aliases", []):
-                registered_commands[alias] = registered_commands[name]
-            return registered_commands[name]
+            return self.add_group(func, **kwargs)
 
         return decorate
 
-    def add_command(self, func: typing.Callable, **kwargs) -> commands.Command:
+    def add_command(self, func: typing.Union[typing.Callable, commands.Command], **kwargs) -> commands.Command:
         """
         Adds a command to the bot. Similar to the ``command`` decorator.
 
         Args:
-            func (:obj:`typing.Callable`): The function to add as a command.
+            func (:obj:`typing.Callable`): The function or command object to register as a command.
 
         Keyword Args:
             **kwargs: See :obj:`~.commands.command` for valid kwargs.
@@ -215,10 +190,16 @@ class BotWithHandler(hikari.Bot):
         Returns:
             :obj:`~.commands.Command`: Command added to the bot.
 
+        Raises:
+            :obj:`NameError`: If the name or an alias of the command being added conflicts with
+                an existing command.
+
         Example:
 
             .. code-block:: python
 
+                import lightbulb
+
                 bot = lightbulb.Bot(token="token_here", prefix="!")
 
                 async def ping(ctx):
@@ -227,31 +208,29 @@ class BotWithHandler(hikari.Bot):
                 bot.add_command(ping)
 
         See Also:
-            :meth:`~.command_handler.BotWithHandler.command`
+            :meth:`~.command_handler.Bot.command`
         """
         if not isinstance(func, commands.Command):
             name = kwargs.get("name", func.__name__)
-            self.commands[name] = commands.Command(
-                func,
-                name,
-                kwargs.get("allow_extra_arguments", True),
-                kwargs.get("aliases", []),
-            )
-            for alias in kwargs.get("aliases", []):
-                self.commands[alias] = self.commands[name]
-        else:
-            name = func.name
-            self.commands[name] = func
-            for alias in func._aliases:
-                self.commands[alias] = func
-        return self.commands[name]
+            cls = kwargs.get("cls", commands.Command)
+            func = cls(func, name, kwargs.get("allow_extra_arguments", True), kwargs.get("aliases", []),)
 
-    def add_group(self, func: typing.Callable, **kwargs) -> commands.Group:
+        if set(self.commands.keys()).intersection({func.name, *func._aliases}):
+            raise NameError(f"Command {func.name} has name or alias already registered.")
+
+        self.commands[func.name] = func
+        for alias in func._aliases:
+            self.commands[alias] = func
+        _LOGGER.debug("new command registered: %s", func.name)
+        return self.commands[func.name]
+
+    def add_group(self, func: typing.Union[typing.Callable, commands.Group], **kwargs) -> commands.Group:
         """
         Adds a command group to the bot. Similar to the ``group`` decorator.
 
         Args:
-            func (:obj:`typing.Callable`): The function to add as a command group.
+            func (Union[ :obj:`typing.Callable`, :obj:`~.commands.Group` ]): The function or group object
+                to register as a command group.
 
         Keyword Args:
             **kwargs: See :obj:`~.commands.group` for valid kwargs.
@@ -259,21 +238,33 @@ class BotWithHandler(hikari.Bot):
         Returns:
             :obj:`~.commands.Group`: Group added to the bot.
 
+        Raises:
+            :obj:`AttributeError`: If the name or an alias of the group being added conflicts with
+                an existing command.
+
         See Also:
-            :meth:`~.command_handler.BotWithHandler.group`
-            :meth:`~.command_handler.BotWithHandler.add_command`
+            :meth:`~.command_handler.Bot.group`
+            :meth:`~.command_handler.Bot.add_command`
         """
-        name = kwargs.get("name", func.__name__)
-        self.commands[name] = commands.Group(
-            func,
-            name,
-            kwargs.get("allow_extra_arguments", True),
-            kwargs.get("aliases", []),
-            insensitive_commands=kwargs.get("insensitive_commands", False),
-        )
-        for alias in kwargs.get("aliases", []):
-            self.commands[alias] = self.commands[name]
-        return self.commands[name]
+        if not isinstance(func, commands.Group):
+            name = kwargs.get("name", func.__name__)
+            cls = kwargs.get("cls", commands.Group)
+            func = cls(
+                func,
+                name,
+                kwargs.get("allow_extra_arguments", True),
+                kwargs.get("aliases", []),
+                insensitive_commands=kwargs.get("insensitive_commands", False),
+            )
+
+        if set(self.commands.keys()).intersection({func.name, *func._aliases}):
+            raise AttributeError(f"Command {func.name} has name or alias already registered.")
+
+        self.commands[func.name] = func
+        for alias in func._aliases:
+            self.commands[alias] = func
+        _LOGGER.debug("new group registered: %s", func.name)
+        return self.commands[func.name]
 
     def add_plugin(self, plugin: plugins.Plugin) -> None:
         """
@@ -284,9 +275,28 @@ class BotWithHandler(hikari.Bot):
 
         Returns:
             ``None``
+
+        Raises:
+            :obj:`NameError`: If the name of the plugin being added conflicts with an
+                existing plugin.
         """
+        if plugin.name in self.plugins:
+            raise NameError(f"A plugin named {plugin.name} is already registered.")
+
         self.plugins[plugin.name] = plugin
-        self.commands.update(plugin.commands)
+        for command in plugin.commands.values():
+            if isinstance(command, commands.Group):
+                self.add_group(command)
+            else:
+                self.add_command(command)
+
+        for event_type, listeners in plugin.listeners.items():
+            for listener in listeners:
+                callback = listener.__get__(plugin, type(plugin))
+                self.subscribe(listener.event_type, callback)
+                _LOGGER.debug("new listener registered: %s (%s)", callback.__name__, listener.event_type.__name__)
+
+        _LOGGER.debug("new plugin registered: %s", plugin.name)
 
     def get_command(self, name: str) -> typing.Optional[commands.Command]:
         """
@@ -323,6 +333,12 @@ class BotWithHandler(hikari.Bot):
             Optional[ :obj:`str` ]: Name of the command that was removed.
         """
         command = self.commands.pop(name)
+        if command is not None:
+            keys_to_remove = [command.name, *command._aliases]
+            keys_to_remove.remove(name)
+            for key in keys_to_remove:
+                self.commands.pop(key)
+            _LOGGER.debug("command removed: %s", command.name)
         return command.name if command is not None else None
 
     def remove_plugin(self, name: str) -> typing.Optional[str]:
@@ -338,7 +354,15 @@ class BotWithHandler(hikari.Bot):
         plugin = self.plugins.pop(name)
         if plugin is not None:
             for k in plugin.commands.keys():
-                self.commands.pop(k)
+                self.remove_command(k)
+
+            for event_type, listeners in plugin.listeners.items():
+                for listener in listeners:
+                    callback = listener.__get__(plugin, type(plugin))
+                    self.unsubscribe(listener.event_type, callback)
+                    _LOGGER.debug("listener removed: %s (%s)", callback.__name__, listener.event_type.__name__)
+
+            _LOGGER.debug("plugin removed: %s", plugin.name)
         return plugin.name if plugin is not None else None
 
     def load_extension(self, extension: str) -> None:
@@ -382,6 +406,7 @@ class BotWithHandler(hikari.Bot):
         else:
             module.load(self)
             self.extensions.append(extension)
+            _LOGGER.debug("new extension loaded: %s", extension)
 
     def unload_extension(self, extension: str) -> None:
         """
@@ -420,13 +445,12 @@ class BotWithHandler(hikari.Bot):
         module = importlib.import_module(extension)
 
         if not hasattr(module, "unload"):
-            raise errors.ExtensionMissingUnload(
-                f"{extension} is missing an unload function"
-            )
+            raise errors.ExtensionMissingUnload(f"{extension} is missing an unload function")
         else:
             module.unload(self)
             self.extensions.remove(extension)
             del sys.modules[extension]
+            _LOGGER.debug("extension unloaded: %s", extension)
 
     def reload_extension(self, extension: str) -> None:
         """
@@ -440,6 +464,7 @@ class BotWithHandler(hikari.Bot):
         Returns:
             ``None``
         """
+        _LOGGER.debug("reloading extension: %s", extension)
         old = sys.modules[extension]
         try:
             self.unload_extension(extension)
@@ -451,9 +476,7 @@ class BotWithHandler(hikari.Bot):
         else:
             del old
 
-    def resolve_arguments(
-        self, message: messages.Message, prefix: str
-    ) -> typing.List[str]:
+    def resolve_arguments(self, message: messages.Message, prefix: str) -> typing.List[str]:
         """
         Resolves the arguments that a command was invoked with from the message containing the invocation.
 
@@ -472,18 +495,14 @@ class BotWithHandler(hikari.Bot):
         string_view = stringview.StringView(message.content[len(prefix) :])
         return string_view.deconstruct_str()
 
-    async def _evaluate_checks(
-        self, command: commands.Command, context: context.Context
-    ):
+    async def _evaluate_checks(self, command: commands.Command, context: context.Context):
         failed_checks = []
 
         for check in command._checks:
             try:
                 if not await check(context):
                     failed_checks.append(
-                        errors.CheckFailure(
-                            f"Check {check.__name__} failed for command {context.invoked_with}"
-                        )
+                        errors.CheckFailure(f"Check {check.__name__} failed for command {context.invoked_with}")
                     )
             except Exception as ex:
                 error = errors.CheckFailure(str(ex))
@@ -491,56 +510,78 @@ class BotWithHandler(hikari.Bot):
                 failed_checks.append(ex)
 
         if len(failed_checks) > 1:
-            raise errors.CheckFailure(
-                "Multiple checks failed: " + ", ".join(str(ex) for ex in failed_checks)
-            )
+            raise errors.CheckFailure("Multiple checks failed: " + ", ".join(str(ex) for ex in failed_checks))
         elif failed_checks:
             raise failed_checks[0]
         return True
 
     async def _invoke_command(
-        self,
-        command: commands.Command,
-        context: context.Context,
-        args: typing.List[str],
+        self, command: commands.Command, context: context.Context, args: typing.List[str],
     ) -> None:
         try:
             if not await self._evaluate_checks(command, context):
                 return
 
-            if (
-                not command.arg_details.has_max_args
-                and len(args) >= command.arg_details.min_args
-            ):
-                await command.invoke(context, *args)
+            (before_asterisk, param_name,) = command.arg_details._args_and_name_before_asterisk()
+            args = self._concatenate_args(args, command)
+
+            if not command.arg_details.has_max_args and len(args) >= command.arg_details.min_args:
+                if param_name is not None:
+                    await command.invoke(
+                        context, *args[:before_asterisk], **{f"{param_name}": args[-1]},
+                    )
+
+                else:
+                    await command.invoke(context, *args)
 
             elif len(args) < command.arg_details.min_args:
                 raise errors.NotEnoughArguments(context.invoked_with)
 
-            elif (
-                len(args) > command.arg_details.max_args
-                and not command._allow_extra_arguments
-            ):
+            elif len(args) > command.arg_details.max_args and not command._allow_extra_arguments:
                 raise errors.TooManyArguments(context.invoked_with)
 
             elif command.arg_details.max_args == 0:
                 await command.invoke(context)
 
             else:
-                await command.invoke(context, *args[: command.arg_details.max_args + 1])
+
+                if param_name is not None:
+                    await command.invoke(
+                        context, *args[:before_asterisk], **{f"{param_name}": args[-1]},
+                    )
+
+                else:
+                    await command.invoke(context, *args[:before_asterisk])
         except errors.CommandError as ex:
             if self.get_listeners(errors.CommandErrorEvent, polymorphic=True):
                 await self.dispatch(errors.CommandErrorEvent(ex, context.message))
             else:
                 raise
 
+    def _concatenate_args(self, args: typing.List[str], command: commands.Command):
+        # Concatenates arguments for last argument (after asterisk sign)
+        new_args = args
+        (before_asterisk, param_name,) = command.arg_details._args_and_name_before_asterisk()
+
+        if before_asterisk < len(args) and not command.arg_details.has_max_args and param_name is not None:
+            new_args = [
+                *args[:before_asterisk],
+                " ".join(args[before_asterisk:]),
+            ]
+
+        return new_args
+
     async def handle(self, event: message.MessageCreateEvent) -> None:
         """
         The message listener that deals with validating the invocation messages. If invocation message
         is valid then it will invoke the relevant command.
 
         Args:
-            event (:obj:`hikari.events.message.MessageCreateEvent`): The message create event containing a possible command invocation.
+            event (:obj:`hikari.events.message.MessageCreateEvent`): The message create event containing
+                a possible command invocation.
+
+        Raises:
+            TypeError: when function's signature has more than 1 argument required after asterisk symbol.
 
         Returns:
             ``None``
@@ -583,7 +624,5 @@ class BotWithHandler(hikari.Bot):
         else:
             new_args = args[1:]
 
-        command_context = context.Context(
-            self, event.message, prefix, invoked_with, invoked_command
-        )
+        command_context = context.Context(self, event.message, prefix, invoked_with, invoked_command)
         await self._invoke_command(invoked_command, command_context, new_args)
