@@ -15,9 +15,37 @@
 #
 # You should have received a copy of the GNU Lesser General Public License
 # along with Lightbulb. If not, see <https://www.gnu.org/licenses/>.
+from __future__ import annotations
+import abc
+import typing
+
 from hikari.events import base
 
-from lightbulb import context
+if not typing.TYPE_CHECKING:
+    import hikari
+
+    from lightbulb import commands
+
+__all__: typing.Final[typing.Tuple[str]] = (
+    "CommandErrorEvent",
+    "ExtensionError",
+    "ExtensionAlreadyLoaded",
+    "ExtensionNotLoaded",
+    "ExtensionMissingLoad",
+    "ExtensionMissingUnload",
+    "CommandError",
+    "CommandNotFound",
+    "NotEnoughArguments",
+    "TooManyArguments",
+    "ConverterFailure",
+    "CommandSyntaxError",
+    "PrematureEOF",
+    "UnclosedQuotes",
+    "CheckFailure",
+    "OnlyInGuild",
+    "OnlyInDM",
+    "NotOwner",
+)
 
 
 class CommandErrorEvent(base.Event):
@@ -42,13 +70,17 @@ class CommandErrorEvent(base.Event):
 
     """
 
-    def __init__(self, error, message) -> None:
+    def __init__(self, error: Exception, message: hikari.Message) -> None:
         self.error = error
         self.traceback = error.__traceback__
         self.message = message
 
 
-class ExtensionError(Exception):
+class LightbulbError(Exception):
+    """Base for any exception raised by lightbulb."""
+
+
+class ExtensionError(LightbulbError):
     """Base exception for errors incurred during the loading and unloading of extensions."""
 
     pass
@@ -78,7 +110,7 @@ class ExtensionMissingUnload(ExtensionError):
     pass
 
 
-class CommandError(Exception):
+class CommandError(LightbulbError):
     """Base exception for errors incurred during handling od commands."""
 
     pass
@@ -121,7 +153,48 @@ class TooManyArguments(CommandError):
         self.invoked_with = invoked_with
 
 
-class UnclosedQuotes(CommandError):
+class ConverterFailure(CommandError):
+    """
+    Exception raised when a converter for a command argument fails.
+    """
+
+    pass
+
+
+class CommandIsOnCooldown(CommandError):
+    """
+    Exception raised when a command is attempted to be run but is currently on cooldown.
+    """
+
+    def __init__(self, text: str, *, command: commands.Command, retry_in: float) -> None:
+        self.text = text
+        self.command = command
+        self.retry_in = retry_in
+
+
+class CommandSyntaxError(CommandError, abc.ABC):
+    """
+    Base error raised if a syntax issue occurs parsing invocation arguments.
+    """
+
+    # Forces the class to be abstract.
+    @abc.abstractmethod
+    def __init__(self):
+        ...
+
+
+class PrematureEOF(CommandSyntaxError):
+    """
+    Error raised if EOF (end of input) was reached, but more content was 
+    expected.
+    """
+
+    def __init__(self) -> None:
+        # Required to override the abstract super init.
+        super().__init__()
+
+
+class UnclosedQuotes(CommandSyntaxError):
     """
     Error raised when no closing quote is found for a quoted argument.
 
@@ -130,6 +203,8 @@ class UnclosedQuotes(CommandError):
     """
 
     def __init__(self, text: str) -> None:
+        # Required to override the abstract super init.
+        super().__init__()
         self.text = text
 
 
