@@ -19,8 +19,11 @@ import typing
 
 from lightbulb import errors
 
+__all__: typing.Final[typing.Tuple[str]] = ("StringView",)
+
 _quotes = {
     '"': '"',
+    "'": "'",
     "‘": "’",
     "‚": "‛",
     "“": "”",
@@ -72,9 +75,11 @@ class StringView:
                 return "".join(buff)
             elif char == "\\":
                 self.index += 1
-                buff.append(self.text[self.index])
-                self.index += 1
-                continue
+                if self.index < len(self.text):
+                    buff.append(self.text[self.index])
+                    self.index += 1
+                    continue
+                raise errors.PrematureEOF()
             else:
                 buff.append(char)
                 self.index += 1
@@ -90,8 +95,9 @@ class StringView:
         a list of all of its arguments. Arguments are defined as words separated by whitespace,
         or as words enclosed in quotation marks.
 
-        Raises ``UnclosedQuotes`` when the :class:`.stringviewer.StringViewer` encounters an EOF 
-        while waiting for a closing quotation mark.
+        Raises:
+            :obj:`~.errors.UnclosedQuotes`: When an unexpected EOF is encountered
+
 
         Returns:
             List[ :obj:`str` ]: The arguments extracted from the string.
